// ***************************************************************************
// ***************************************************************************
// Copyright 2011(c) Analog Devices, Inc.
// 
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//     - Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     - Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in
//       the documentation and/or other materials provided with the
//       distribution.
//     - Neither the name of Analog Devices, Inc. nor the names of its
//       contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//     - The use of this software may or may not infringe the patent rights
//       of one or more patent holders.  This license does not release you
//       from the requirement that you obtain separate licenses from these
//       patent holders to use this software.
//     - Use of the software either in source or binary form, must be run
//       on or directly connected to an Analog Devices Inc. component.
//    
// THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED.
//
// IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, INTELLECTUAL PROPERTY
// RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
// THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// ***************************************************************************
// ***************************************************************************
// ***************************************************************************
// ***************************************************************************

`timescale 1ns/100ps

module cf_ad9250 (

  // jesd interface (ref_clk is the sample clock, rx_clk is ref_clk/2)

  rx_clk,
  rx_data0,
  rx_data1,
  ref_clk,

  // dma interface

  dma_clk,
  dma_valid,
  dma_data,
  dma_be,
  dma_last,
  dma_ready,

  // processor interface

  up_rstn,
  up_clk,
  up_sel,
  up_rwn,
  up_addr,
  up_wdata,
  up_rdata,
  up_ack,

  // debug interface (chipscope)

  adc_mon_valid,
  adc_mon_data);

  // jesd interface (ref_clk is the sample clock, rx_clk is ref_clk/2)

  input           rx_clk;
  input   [31:0]  rx_data0;
  input   [31:0]  rx_data1;
  input           ref_clk;

  // dma interface

  input           dma_clk;
  output          dma_valid;
  output  [63:0]  dma_data;
  output  [ 7:0]  dma_be;
  output          dma_last;
  input           dma_ready;

  // processor interface

  input           up_rstn;
  input           up_clk;
  input           up_sel;
  input           up_rwn;
  input   [ 4:0]  up_addr;
  input   [31:0]  up_wdata;
  output  [31:0]  up_rdata;
  output          up_ack;

  // debug interface (chipscope)

  output          adc_mon_valid;
  output  [31:0]  adc_mon_data;

  reg             up_capture = 'd0;
  reg     [15:0]  up_capture_count = 'd0;
  reg             up_dma_unf_hold = 'd0;
  reg             up_dma_ovf_hold = 'd0;
  reg             up_dma_status = 'd0;
  reg     [ 1:0]  up_adc_or_hold = 'd0;
  reg     [ 1:0]  up_adc_pn_oos_hold = 'd0;
  reg     [ 1:0]  up_adc_pn_err_hold = 'd0;
  reg     [ 1:0]  up_pn_type = 'd0;
  reg             up_data_sel = 'd0;
  reg     [31:0]  up_rdata = 'd0;
  reg             up_sel_d = 'd0;
  reg             up_sel_2d = 'd0;
  reg             up_ack = 'd0;
  reg             up_dma_ovf_m1 = 'd0;
  reg             up_dma_ovf_m2 = 'd0;
  reg             up_dma_ovf = 'd0;
  reg             up_dma_unf_m1 = 'd0;
  reg             up_dma_unf_m2 = 'd0;
  reg             up_dma_unf = 'd0;
  reg             up_dma_complete_m1 = 'd0;
  reg             up_dma_complete_m2 = 'd0;
  reg             up_dma_complete_m3 = 'd0;
  reg             up_dma_complete = 'd0;
  reg     [ 1:0]  up_adc_or_m1 = 'd0;
  reg     [ 1:0]  up_adc_or_m2 = 'd0;
  reg     [ 1:0]  up_adc_or = 'd0;
  reg     [ 1:0]  up_adc_pn_oos_m1 = 'd0;
  reg     [ 1:0]  up_adc_pn_oos_m2 = 'd0;
  reg     [ 1:0]  up_adc_pn_oos = 'd0;
  reg     [ 1:0]  up_adc_pn_err_m1 = 'd0;
  reg     [ 1:0]  up_adc_pn_err_m2 = 'd0;
  reg     [ 1:0]  up_adc_pn_err = 'd0;

  wire            up_wr_s;
  wire            up_ack_s;
  wire            dma_ovf_s;
  wire            dma_unf_s;
  wire            dma_complete_s;
  wire            adc_valid_s;
  wire    [63:0]  adc_data_s;
  wire    [ 1:0]  adc_or_s;
  wire    [ 1:0]  adc_pn_oos_s;
  wire    [ 1:0]  adc_pn_err_s;

  // processor interface (refer to the regmap.txt file in the pcores directory for
  // a description of these registers.

  assign up_wr_s = up_sel & ~up_rwn;
  assign up_ack_s = up_sel_d & ~up_sel_2d;

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      up_capture <= 'd0;
      up_capture_count <= 'd0;
      up_dma_unf_hold <= 'd0;
      up_dma_ovf_hold <= 'd0;
      up_dma_status <= 'd0;
      up_adc_or_hold <= 'd0;
      up_adc_pn_oos_hold <= 'd0;
      up_adc_pn_err_hold <= 'd0;
      up_pn_type <= 'd0;
      up_data_sel <= 'd0;
    end else begin
      if ((up_addr == 5'h03) && (up_wr_s == 1'b1)) begin
        up_capture <= up_wdata[16];
        up_capture_count <= up_wdata[15:0];
      end
      if (up_dma_unf == 1'b1) begin
        up_dma_unf_hold <= 1'b1;
      end else if ((up_addr == 5'h04) && (up_wr_s == 1'b1)) begin
        up_dma_unf_hold <= up_dma_unf_hold & ~up_wdata[2];
      end
      if (up_dma_ovf == 1'b1) begin
        up_dma_ovf_hold <= 1'b1;
      end else if ((up_addr == 5'h04) && (up_wr_s == 1'b1)) begin
        up_dma_ovf_hold <= up_dma_ovf_hold & ~up_wdata[2];
      end
      if (up_dma_complete == 1'b1) begin
        up_dma_status <= 1'b0;
      end else if ((up_addr == 5'h03) && (up_wr_s == 1'b1) && (up_dma_status == 1'b0)) begin
        up_dma_status <= up_wdata[16];
      end
      if (up_adc_or[0] == 1'b1) begin
        up_adc_or_hold[0] <= 1'b1;
      end else if ((up_addr == 5'h05) && (up_wr_s == 1'b1)) begin
        up_adc_or_hold[0] <= up_adc_or_hold[0] & ~up_wdata[0];
      end
      if (up_adc_or[1] == 1'b1) begin
        up_adc_or_hold[1] <= 1'b1;
      end else if ((up_addr == 5'h05) && (up_wr_s == 1'b1)) begin
        up_adc_or_hold[1] <= up_adc_or_hold[1] & ~up_wdata[1];
      end
      if (up_adc_pn_oos[0] == 1'b1) begin
        up_adc_pn_oos_hold[0] <= 1'b1;
      end else if ((up_addr == 5'h05) && (up_wr_s == 1'b1)) begin
        up_adc_pn_oos_hold[0] <= up_adc_pn_oos_hold[0] & ~up_wdata[2];
      end
      if (up_adc_pn_oos[1] == 1'b1) begin
        up_adc_pn_oos_hold[1] <= 1'b1;
      end else if ((up_addr == 5'h05) && (up_wr_s == 1'b1)) begin
        up_adc_pn_oos_hold[1] <= up_adc_pn_oos_hold[1] & ~up_wdata[3];
      end
      if (up_adc_pn_err[0] == 1'b1) begin
        up_adc_pn_err_hold[0] <= 1'b1;
      end else if ((up_addr == 5'h05) && (up_wr_s == 1'b1)) begin
        up_adc_pn_err_hold[0] <= up_adc_pn_err_hold[0] & ~up_wdata[4];
      end
      if (up_adc_pn_err[1] == 1'b1) begin
        up_adc_pn_err_hold[1] <= 1'b1;
      end else if ((up_addr == 5'h05) && (up_wr_s == 1'b1)) begin
        up_adc_pn_err_hold[1] <= up_adc_pn_err_hold[1] & ~up_wdata[5];
      end
      if ((up_addr == 5'h09) && (up_wr_s == 1'b1)) begin
        up_pn_type <= up_wdata[1:0];
      end
      if ((up_addr == 5'h0a) && (up_wr_s == 1'b1)) begin
        up_data_sel <= up_wdata[0];
      end
    end
  end

  // processor read interface

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      up_rdata <= 'd0;
      up_sel_d <= 'd0;
      up_sel_2d <= 'd0;
      up_ack <= 'd0;
    end else begin
      case (up_addr)
        5'h00: up_rdata <= 32'h00010061;
        5'h03: up_rdata <= {15'd0, up_capture, up_capture_count};
        5'h04: up_rdata <= {29'd0, up_dma_unf_hold, up_dma_ovf_hold, up_dma_status};
        5'h05: up_rdata <= {26'd0, up_adc_pn_err_hold, up_adc_pn_oos_hold, up_adc_or_hold};
        5'h09: up_rdata <= {30'd0, up_pn_type};
        5'h0a: up_rdata <= {31'd0, up_data_sel};
        default: up_rdata <= 0;
      endcase
      up_sel_d <= up_sel;
      up_sel_2d <= up_sel_d;
      up_ack <= up_ack_s;
    end
  end

  // overflow, underflow and dma status (clock domain transfers)

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      up_dma_ovf_m1 <= 'd0;
      up_dma_ovf_m2 <= 'd0;
      up_dma_ovf <= 'd0;
      up_dma_unf_m1 <= 'd0;
      up_dma_unf_m2 <= 'd0;
      up_dma_unf <= 'd0;
      up_dma_complete_m1 <= 'd0;
      up_dma_complete_m2 <= 'd0;
      up_dma_complete_m3 <= 'd0;
      up_dma_complete <= 'd0;
    end else begin
      up_dma_ovf_m1 <= dma_ovf_s;
      up_dma_ovf_m2 <= up_dma_ovf_m1;
      up_dma_ovf <= up_dma_ovf_m2;
      up_dma_unf_m1 <= dma_unf_s;
      up_dma_unf_m2 <= up_dma_unf_m1;
      up_dma_unf <= up_dma_unf_m2;
      up_dma_complete_m1 <= dma_complete_s;
      up_dma_complete_m2 <= up_dma_complete_m1;
      up_dma_complete_m3 <= up_dma_complete_m2;
      up_dma_complete <= up_dma_complete_m3 ^ up_dma_complete_m2;
    end
  end

  // pn err, oos and over range (clock domain transfers)

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      up_adc_or_m1 <= 'd0;
      up_adc_or_m2 <= 'd0;
      up_adc_or <= 'd0;
      up_adc_pn_oos_m1 <= 'd0;
      up_adc_pn_oos_m2 <= 'd0;
      up_adc_pn_oos <= 'd0;
      up_adc_pn_err_m1 <= 'd0;
      up_adc_pn_err_m2 <= 'd0;
      up_adc_pn_err <= 'd0;
    end else begin
      up_adc_or_m1 <= adc_or_s;
      up_adc_or_m2 <= up_adc_or_m1;
      up_adc_or <= up_adc_or_m2;
      up_adc_pn_oos_m1 <= adc_pn_oos_s;
      up_adc_pn_oos_m2 <= up_adc_pn_oos_m1;
      up_adc_pn_oos <= up_adc_pn_oos_m2;
      up_adc_pn_err_m1 <= adc_pn_err_s;
      up_adc_pn_err_m2 <= up_adc_pn_err_m1;
      up_adc_pn_err <= up_adc_pn_err_m2;
    end
  end

  // dma interface

  cf_dma_wr i_dma_wr (
    .adc_clk (ref_clk),
    .adc_valid (adc_valid_s),
    .adc_data (adc_data_s),
    .adc_master_capture (up_capture),
    .dma_clk (dma_clk),
    .dma_valid (dma_valid),
    .dma_data (dma_data),
    .dma_be (dma_be),
    .dma_last (dma_last),
    .dma_ready (dma_ready),
    .dma_ovf (dma_ovf_s),
    .dma_unf (dma_unf_s),
    .dma_complete (dma_complete_s),
    .up_capture_count (up_capture_count),
    .dma_dbg_data (),
    .dma_dbg_trigger (),
    .adc_dbg_data (),
    .adc_dbg_trigger ());

  // adc interface

  cf_adc_if i_adc_if (
    .rx_clk (rx_clk),
    .rx_data0 (rx_data0),
    .rx_data1 (rx_data1),
    .ref_clk (ref_clk),
    .adc_valid (adc_valid_s),
    .adc_data (adc_data_s),
    .adc_or (adc_or_s),
    .adc_pn_oos (adc_pn_oos_s),
    .adc_pn_err (adc_pn_err_s),
    .up_pn_type (up_pn_type),
    .up_data_sel (up_data_sel),
    .adc_mon_valid (adc_mon_valid),
    .adc_mon_data (adc_mon_data));

endmodule

// ***************************************************************************
// ***************************************************************************
