// ***************************************************************************
// ***************************************************************************
// Copyright 2011(c) Analog Devices, Inc.
// 
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//     - Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     - Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in
//       the documentation and/or other materials provided with the
//       distribution.
//     - Neither the name of Analog Devices, Inc. nor the names of its
//       contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//     - The use of this software may or may not infringe the patent rights
//       of one or more patent holders.  This license does not release you
//       from the requirement that you obtain separate licenses from these
//       patent holders to use this software.
//     - Use of the software either in source or binary form, must be run
//       on or directly connected to an Analog Devices Inc. component.
//    
// THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED.
//
// IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, INTELLECTUAL PROPERTY
// RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
// THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// ***************************************************************************
// ***************************************************************************

`timescale 1ns/100ps

module jesd204b_rx2_top (

  // Physical interface, except for data, all other inputs/outputs must be buffered
  // outside this module. Refer to the utility cores for the jesd buffers.

  ref_clk_in,
  rxdata_p,
  rxdata_n,
  rxsync,
  rxsysref,

  // these are the outputs of the core

  rx_clk,
  rx_data0,
  rx_data1,

  // master interface and status signals

  mb_req,
  mb_ack,
  mb_addr,
  mb_data,
  mb_done,
  mb_error,
  mb_readyn,

  // processor interface (see regmap.txt at the pcore directory for details)

  up_rstn,
  up_clk,
  up_sel,
  up_rwn,
  up_addr,
  up_wdata,
  up_rdata,
  up_ack,

  // debug interface (chipscope)

  es_dbg_data,
  es_dbg_trigger,

  // es debug data (for chipscope)

  mb_dbg_data,
  mb_dbg_trigger,

  // debug interface (chipscope)

  debug_data,
  debug_trigger);

  parameter C_CF_BUFTYPE = 0;

  // Physical interface, except for data, all other inputs/outputs must be buffered
  // outside this module. Refer to the utility cores for the jesd buffers.

  input           ref_clk_in;
  input   [ 1:0]  rxdata_p;
  input   [ 1:0]  rxdata_n;
  output          rxsync;
  output          rxsysref;

  // these are the outputs of the jesd core, this interface is same as the xilinx ip

  output          rx_clk;
  output  [31:0]  rx_data0;
  output  [31:0]  rx_data1;

  // master interface and status signals

  output          mb_req;
  input           mb_ack;
  output  [31: 0] mb_addr;
  output  [31: 0] mb_data;
  input           mb_done;
  input           mb_error;
  input           mb_readyn;

  // processor interface (see regmap.txt at the pcore directory for details)

  input           up_rstn;
  input           up_clk;
  input           up_sel;
  input           up_rwn;
  input   [ 4:0]  up_addr;
  input   [31:0]  up_wdata;
  output  [31:0]  up_rdata;
  output          up_ack;

  // debug interface (chipscope)

  output [227:0]  es_dbg_data;
  output  [ 7:0]  es_dbg_trigger;

  // es debug data (for chipscope)

  output  [83:0]  mb_dbg_data;
  output  [ 7:0]  mb_dbg_trigger;

  // debug interface (chipscope)

  output [199:0]  debug_data;
  output  [ 9:0]  debug_trigger;

  reg             up_lanesync_enb = 'd0;
  reg             up_scr_enb = 'd0;
  reg             up_sysref_enb = 'd0;
  reg             up_err_disb = 'd0;
  reg     [ 4:0]  up_frmcnt = 'd0;
  reg     [ 7:0]  up_bytecnt = 'd0;
  reg     [12:0]  up_bufdelay = 'd0;
  reg             up_gtx_rstn = 'd0;
  reg             up_jesd204b_rstn = 'd0;
  reg     [ 1:0]  up_test_mode = 'd0;
  reg     [ 1:0]  up_lanesel = 'd0;
  reg             up_es_mode = 'd0;
  reg             up_es_start = 'd0;
  reg     [ 4:0]  up_prescale = 'd0;
  reg     [ 7:0]  up_voffset_step = 'd0;
  reg     [ 7:0]  up_voffset_max = 'd0;
  reg     [ 7:0]  up_voffset_min = 'd0;
  reg     [11:0]  up_hoffset_max = 'd0;
  reg     [11:0]  up_hoffset_min = 'd0;
  reg     [11:0]  up_hoffset_step = 'd0;
  reg     [31:0]  up_startaddr = 'd0;
  reg     [15:0]  up_hsize = 'd0;
  reg     [15:0]  up_hmax = 'd0;
  reg     [15:0]  up_hmin = 'd0;
  reg             up_mb_ovf_hold = 'd0;
  reg             up_mb_unf_hold = 'd0;
  reg     [ 1:0]  up_mb_status_hold = 'd0;
  reg     [31:0]  up_rdata = 'd0;
  reg             up_sel_d = 'd0;
  reg             up_sel_2d = 'd0;
  reg             up_ack = 'd0;
  reg             up_drp_en_0 = 'd0;
  reg             up_drp_en_1 = 'd0;
  reg             up_drp_we = 'd0;
  reg     [ 8:0]  up_drp_addr = 'd0;
  reg     [15:0]  up_drp_wdata = 'd0;
  reg     [15:0]  up_drp_rdata = 'd0;
  reg             up_drp_ready = 'd0;
  reg             jesd204b_status_m1 = 'd0;
  reg             jesd204b_status = 'd0;
  reg     [31:0]  jesd204b_init_data0 = 'd0;
  reg     [31:0]  jesd204b_init_data1 = 'd0;
  reg     [31:0]  jesd204b_init_data2 = 'd0;
  reg     [31:0]  jesd204b_init_data3 = 'd0;
  reg     [31:0]  jesd204b_test_mfcnt = 'd0;
  reg     [31:0]  jesd204b_test_ilacnt = 'd0;
  reg     [31:0]  jesd204b_test_errcnt = 'd0;
  reg             rx_sysref_enb = 'd0;
  reg             rx_sysref_enb_d = 'd0;
  reg             rxsysref = 'd0;
  reg             rx_sync_enb = 'd0;
  reg             rxsync = 'd0;
  reg             rxstatus = 'd0;
 
  wire            up_wr_s;
  wire            up_ack_s;
  wire            jesd204b_pre_s;
  wire            gtx_pre_s;
  wire            jesd204b_rst_s;
  wire            gtx_rst_s;
  wire    [ 3:0]  jesd204b_rxsync_s;
  wire    [15:0]  jesd204b_bufcnt_s;
  wire    [ 3:0]  jesd204b_rxsof_s;
  wire    [ 3:0]  jesd204b_rxeof_s;
  wire    [ 3:0]  jesd204b_rxferr0_s;
  wire    [ 3:0]  jesd204b_rxferr1_s;
  wire    [31:0]  jesd204b_rxfdata0_s;
  wire    [31:0]  jesd204b_rxfdata1_s;
  wire    [63:0]  jesd204b_test_errcnt_s;
  wire    [63:0]  jesd204b_test_ilacnt_s;
  wire    [63:0]  jesd204b_test_mfcnt_s;
  wire    [63:0]  jesd204b_init_data0_s;
  wire    [63:0]  jesd204b_init_data1_s;
  wire    [63:0]  jesd204b_init_data2_s;
  wire    [63:0]  jesd204b_init_data3_s;
  wire            gtx_rxrstdone_s;
  wire            gtx_rxclk_s;
  wire    [63:0]  gtx_rxdata_s;
  wire    [ 7:0]  gtx_rxcharisk_s;
  wire    [ 7:0]  gtx_rxdisperr_s;
  wire    [ 7:0]  gtx_rxnotintable_s;
  wire            gtx_rxencommaalign_s;
  wire            gtx_rxenchanbond_s;
  wire            gtx_rxchanbondseq_s;
  wire            up_drp_en_s;
  wire            up_drp_we_s;
  wire    [ 8:0]  up_drp_addr_s;
  wire    [15:0]  up_drp_wdata_s;
  wire    [15:0]  up_drp_rdata_0_s;
  wire            up_drp_ready_0_s;
  wire    [15:0]  up_drp_rdata_1_s;
  wire            up_drp_ready_1_s;
  wire            es_valid_s;
  wire            es_sos_s;
  wire            es_eos_s;
  wire    [31:0]  es_data_s;
  wire            es_state_s;
  wire            mb_ovf_s;
  wire            mb_unf_s;
  wire    [ 1:0]  mb_status_s;
  wire            mb_state_s;
  wire    [ 1:0]  rx_sysref_enb_s;
  wire    [ 1:0]  rx_sync_enb_s;
  wire    [ 7:0]  rx_fsm_s;
  wire    [ 1:0]  rx_align_err_s;

  // debug interface

  assign debug_trigger[9] = rxsysref;
  assign debug_trigger[8] = rxsync;

  assign debug_data[199:196] = jesd204b_rxsof_s;
  assign debug_data[195:192] = jesd204b_rxeof_s;
  assign debug_data[191:188] = jesd204b_rxferr0_s;
  assign debug_data[187:184] = jesd204b_rxferr1_s;
  assign debug_data[183:152] = jesd204b_rxfdata0_s;
  assign debug_data[151:120] = jesd204b_rxfdata1_s;
  assign debug_data[119:119] = rxsysref;
  assign debug_data[118:118] = rxsync;
  assign debug_data[117:117] = jesd204b_rst_s;
  assign debug_data[116:116] = gtx_rst_s;
  assign debug_data[115:112] = jesd204b_rxsync_s;

  // processor interface, see the regmap.txt for a description of the registers

  assign up_wr_s = up_sel & ~up_rwn;
  assign up_ack_s = up_sel_d & ~up_sel_2d;

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      up_lanesync_enb <= 'd0;
      up_scr_enb <= 'd0;
      up_sysref_enb <= 'd0;
      up_err_disb <= 'd0;
      up_frmcnt <= 'd0;
      up_bytecnt <= 'd0;
      up_bufdelay <= 'd0;
      up_gtx_rstn <= 'd0;
      up_jesd204b_rstn <= 'd0;
      up_test_mode <= 'd0;
      up_lanesel <= 'd0;
      up_es_mode <= 'd0;
      up_es_start <= 'd0;
      up_prescale <= 'd0;
      up_voffset_step <= 'd0;
      up_voffset_max <= 'd0;
      up_voffset_min <= 'd0;
      up_hoffset_max <= 'd0;
      up_hoffset_min <= 'd0;
      up_hoffset_step <= 'd0;
      up_startaddr <= 'd0;
      up_hsize <= 'd0;
      up_hmin <= 'd0;
      up_hmax <= 'd0;
      up_mb_ovf_hold <= 'd0;
      up_mb_unf_hold <= 'd0;
      up_mb_status_hold <= 'd0;
    end else begin
      if ((up_addr == 5'h01) && (up_wr_s == 1'b1)) begin
        up_lanesync_enb <= up_wdata[3];
        up_scr_enb <= up_wdata[2];
        up_sysref_enb <= up_wdata[1];
        up_err_disb <= up_wdata[0];
      end
      if ((up_addr == 5'h03) && (up_wr_s == 1'b1)) begin
        up_frmcnt <= up_wdata[12:8];
        up_bytecnt <= up_wdata[7:0];
      end
      if ((up_addr == 5'h04) && (up_wr_s == 1'b1)) begin
        up_bufdelay <= up_wdata[12:0];
      end
      if ((up_addr == 5'h05) && (up_wr_s == 1'b1)) begin
        up_gtx_rstn <= up_wdata[5];
        up_jesd204b_rstn <= up_wdata[4];
        up_test_mode <= up_wdata[3:2];
        up_lanesel <= up_wdata[1:0];
      end
      if ((up_addr == 5'h10) && (up_wr_s == 1'b1)) begin
        up_es_mode <= up_wdata[1];
        up_es_start <= up_wdata[0];
      end
      if ((up_addr == 5'h11) && (up_wr_s == 1'b1)) begin
        up_prescale <= up_wdata[4:0];
      end
      if ((up_addr == 5'h12) && (up_wr_s == 1'b1)) begin
        up_voffset_step <= up_wdata[23:16];
        up_voffset_max <= up_wdata[15:8];
        up_voffset_min <= up_wdata[7:0];
      end
      if ((up_addr == 5'h13) && (up_wr_s == 1'b1)) begin
        up_hoffset_max <= up_wdata[27:16];
        up_hoffset_min <= up_wdata[11:0];
      end
      if ((up_addr == 5'h14) && (up_wr_s == 1'b1)) begin
        up_hoffset_step <= up_wdata[11:0];
      end
      if ((up_addr == 5'h15) && (up_wr_s == 1'b1)) begin
        up_startaddr <= up_wdata;
      end
      if ((up_addr == 5'h16) && (up_wr_s == 1'b1)) begin
        up_hsize <= up_wdata[15:0];
      end
      if ((up_addr == 5'h17) && (up_wr_s == 1'b1)) begin
        up_hmax <= up_wdata[31:16];
        up_hmin <= up_wdata[15:0];
      end
      if ((up_addr == 5'h18) && (up_wr_s == 1'b1)) begin
        up_mb_ovf_hold <= up_mb_ovf_hold & ~up_wdata[5];
        up_mb_unf_hold <= up_mb_unf_hold & ~up_wdata[4];
        up_mb_status_hold[1] <= up_mb_status_hold[1] & ~up_wdata[3];
        up_mb_status_hold[0] <= up_mb_status_hold[0] & ~up_wdata[2];
      end else begin
        up_mb_ovf_hold <= up_mb_ovf_hold | mb_ovf_s;
        up_mb_unf_hold <= up_mb_unf_hold | mb_unf_s;
        up_mb_status_hold[1] <= up_mb_status_hold[1] | mb_status_s[1];
        up_mb_status_hold[0] <= up_mb_status_hold[0] | mb_status_s[0];
      end
    end
  end

  // processor read interface

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      up_rdata <= 'd0;
      up_sel_d <= 'd0;
      up_sel_2d <= 'd0;
      up_ack <= 'd0;
    end else begin
      case (up_addr)
        5'h00: up_rdata <= 32'h00010062;
        5'h01: up_rdata <= {28'd0, up_lanesync_enb, up_scr_enb, up_sysref_enb, up_err_disb};
        5'h02: up_rdata <= {31'd0, jesd204b_status};
        5'h03: up_rdata <= {19'd0, up_frmcnt, up_bytecnt};
        5'h04: up_rdata <= {19'd0, up_bufdelay};
        5'h05: up_rdata <= {26'd0, up_gtx_rstn, up_jesd204b_rstn, up_test_mode, up_lanesel};
        5'h06: up_rdata <= {16'd0, jesd204b_bufcnt_s};
        5'h07: up_rdata <= jesd204b_init_data0;
        5'h08: up_rdata <= jesd204b_init_data1;
        5'h09: up_rdata <= jesd204b_init_data2;
        5'h0a: up_rdata <= jesd204b_init_data3;
        5'h0b: up_rdata <= jesd204b_test_mfcnt;
        5'h0c: up_rdata <= jesd204b_test_ilacnt;
        5'h0d: up_rdata <= jesd204b_test_errcnt;
        5'h10: up_rdata <= {30'd0, up_es_mode, up_es_start};
        5'h11: up_rdata <= {27'd0, up_prescale};
        5'h12: up_rdata <= {8'd0, up_voffset_step, up_voffset_max, up_voffset_min};
        5'h13: up_rdata <= {4'd0, up_hoffset_max, 4'd0, up_hoffset_min};
        5'h14: up_rdata <= {20'd0, up_hoffset_step};
        5'h15: up_rdata <= up_startaddr;
        5'h16: up_rdata <= {16'd0, up_hsize};
        5'h17: up_rdata <= {up_hmax, up_hmin};
        5'h18: up_rdata <= {26'd0, up_mb_ovf_hold, up_mb_unf_hold, up_mb_status_hold,
                            mb_state_s, es_state_s};
        default: up_rdata <= 0;
      endcase
      up_sel_d <= up_sel;
      up_sel_2d <= up_sel_d;
      up_ack <= up_ack_s;
    end
  end

  // drp mux based on lane select

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      up_drp_en_0 <= 'd0;
      up_drp_en_1 <= 'd0;
      up_drp_we <= 'd0;
      up_drp_addr <= 'd0;
      up_drp_wdata <= 'd0;
      up_drp_rdata <= 'd0;
      up_drp_ready <= 'd0;
    end else begin
      up_drp_en_0 <= (up_lanesel == 2'b00) ? up_drp_en_s : 1'b0;
      up_drp_en_1 <= (up_lanesel == 2'b01) ? up_drp_en_s : 1'b0;
      up_drp_we <= up_drp_we_s;
      up_drp_addr <= up_drp_addr_s;
      up_drp_wdata <= up_drp_wdata_s;
      case (up_lanesel)
        2'b01: begin
          up_drp_rdata <= up_drp_rdata_1_s;
          up_drp_ready <= up_drp_ready_1_s;
        end
        default: begin
          up_drp_rdata <= up_drp_rdata_0_s;
          up_drp_ready <= up_drp_ready_0_s;
        end
      endcase
    end
  end

  // since the number of register spaces is limited, the gtx status/control signals are
  // multiplexed per lane (a single lane instantiates one gtx)

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      jesd204b_status_m1 <= 'd0;
      jesd204b_status <= 'd0;
      jesd204b_init_data0 <= 'd0;
      jesd204b_init_data1 <= 'd0;
      jesd204b_init_data2 <= 'd0;
      jesd204b_init_data3 <= 'd0;
      jesd204b_test_mfcnt <= 'd0;
      jesd204b_test_ilacnt <= 'd0;
      jesd204b_test_errcnt <= 'd0;
    end else begin
      jesd204b_status_m1 <= rxstatus;
      jesd204b_status <= jesd204b_status_m1;
      case (up_lanesel)
        2'b01: begin
          jesd204b_init_data0 <= jesd204b_init_data0_s[63:32];
          jesd204b_init_data1 <= jesd204b_init_data1_s[63:32];
          jesd204b_init_data2 <= jesd204b_init_data2_s[63:32];
          jesd204b_init_data3 <= jesd204b_init_data3_s[63:32];
          jesd204b_test_mfcnt <= jesd204b_test_mfcnt_s[63:32];
          jesd204b_test_ilacnt <= jesd204b_test_ilacnt_s[63:32];
          jesd204b_test_errcnt <= jesd204b_test_errcnt_s[63:32];
        end
        default: begin
          jesd204b_init_data0 <= jesd204b_init_data0_s[31:0];
          jesd204b_init_data1 <= jesd204b_init_data1_s[31:0];
          jesd204b_init_data2 <= jesd204b_init_data2_s[31:0];
          jesd204b_init_data3 <= jesd204b_init_data3_s[31:0];
          jesd204b_test_mfcnt <= jesd204b_test_mfcnt_s[31:0];
          jesd204b_test_ilacnt <= jesd204b_test_ilacnt_s[31:0];
          jesd204b_test_errcnt <= jesd204b_test_errcnt_s[31:0];
        end
      endcase
    end
  end

  // generate sysref and sync controlled by the monitor state machine

  always @(posedge rx_clk) begin
    if (jesd204b_rst_s == 1'b1) begin
      rx_sysref_enb <= 'd0;
      rx_sysref_enb_d <= 'd0;
      rxsysref <= 'd0;
      rx_sync_enb <= 'd0;
      rxsync <= 'd0;
      rxstatus <= 'd0;
    end else begin
      rx_sysref_enb <= (rx_sysref_enb_s == 2'b11) ? 1'b1 : 1'b0;
      rx_sysref_enb_d <= rx_sysref_enb;
      rxsysref <= rx_sysref_enb & ~rx_sysref_enb_d;
      rx_sync_enb <= (rx_sync_enb_s == 2'b11) ? 1'b1 : 1'b0;
      rxsync <= (jesd204b_rxsync_s == 4'b1111) ? rx_sync_enb : 1'b0;
      rxstatus <= ((rx_fsm_s == 8'h44) && (rx_align_err_s == 2'h0)) ? 1'b1 : 1'b0;
    end
  end

  // the gtx is held on reset until processor is up.
  // the jesd core is held on reset until after gtx is locked and running

  assign jesd204b_pre_s = ~(gtx_rxrstdone_s & up_jesd204b_rstn);
  assign gtx_pre_s = ~up_gtx_rstn;

  FDPE #(.INIT(1'b1)) i_jesd204b_rst_reg (
    .CE (1'b1),
    .D (1'b0),
    .PRE (jesd204b_pre_s),
    .C (rx_clk),
    .Q (jesd204b_rst_s));

  FDPE #(.INIT(1'b1)) i_gtx_rst_reg (
    .CE (1'b1),
    .D (1'b0),
    .PRE (gtx_pre_s),
    .C (up_clk),
    .Q (gtx_rst_s));

  // this is the receive clock buffer, (the user clock as per xilinx definitions)

  BUFG i_adc_clk_bufg (
    .I (gtx_rxclk_s),
    .O (rx_clk));

  // jesd core (xilinx), documentation and pin definitions are available on-line

  jesd204b_rx2 i_jesd204b_rx2 (
    .clk (rx_clk),
    .rst (jesd204b_rst_s),
    .rxdata (gtx_rxdata_s),
    .rxcharisk (gtx_rxcharisk_s),
    .rxdisperr (gtx_rxdisperr_s),
    .rxnotintable (gtx_rxnotintable_s),
    .encommaalign (gtx_rxencommaalign_s),
    .enchanbond (gtx_rxenchanbond_s),
    .chanbondseq (gtx_rxchanbondseq_s),
    .sysref_in (rxsysref),
    .sysref_out (),
    .rx_sync (jesd204b_rxsync_s),
    .start_of_frame (jesd204b_rxsof_s),
    .end_of_frame (jesd204b_rxeof_s),
    .frame_error ({jesd204b_rxferr1_s, jesd204b_rxferr0_s}),
    .rxdataout ({jesd204b_rxfdata1_s, jesd204b_rxfdata0_s}),
    .support_lane_sync (up_lanesync_enb),
    .scram_enable (up_scr_enb),
    .sysref_always (up_sysref_enb),
    .disable_error_reporting (up_err_disb),
    .octets_per_frame (up_bytecnt),
    .frames_per_multiframe (up_frmcnt),
    .rx_buffer_delay (up_bufdelay),
    .test_modes (up_test_mode),
    .rx_buffer_adjust (jesd204b_bufcnt_s),
    .test_err_count (jesd204b_test_errcnt_s),
    .test_ila_count (jesd204b_test_ilacnt_s),
    .test_mf_count (jesd204b_test_mfcnt_s),
    .init0 (jesd204b_init_data0_s),
    .init1 (jesd204b_init_data1_s),
    .init2 (jesd204b_init_data2_s),
    .init3 (jesd204b_init_data3_s),
    .rxstatus ());

  // jesd alignment- per lane (all lanes share the same sof/eof)

  cf_jesd_align_2 i_align_2_1 (
    .rx_clk (rx_clk),
    .rx_sof (jesd204b_rxsof_s),
    .rx_eof (jesd204b_rxeof_s),
    .rx_ferr (jesd204b_rxferr1_s),
    .rx_fdata (jesd204b_rxfdata1_s),
    .rx_err (rx_align_err_s[1]),
    .rx_data (rx_data1));

  cf_jesd_align_2 i_align_2_0 (
    .rx_clk (rx_clk),
    .rx_sof (jesd204b_rxsof_s),
    .rx_eof (jesd204b_rxeof_s),
    .rx_ferr (jesd204b_rxferr0_s),
    .rx_fdata (jesd204b_rxfdata0_s),
    .rx_err (rx_align_err_s[0]),
    .rx_data (rx_data0));

  // gtx core (xilinx), this is the key element in this interface (though under-stated
  // by xilinx. The files generated by the coregen are hard to follow through and is
  // a mix of testbench and rtl, so a little hand-editing is done after being
  // generated by coregen, but most of it is the same.

  jesd204b_rx2_gtwizard_v2_1_top # (.EXAMPLE_SIM_GTRESET_SPEEDUP("TRUE")) i_gtx_rx2 (
    .txreset (gtx_rst_s),
    .rxreset (gtx_rst_s),
    .refclk (ref_clk_in),
    .txp_out (),
    .txn_out (),
    .rxp_in (rxdata_p),
    .rxn_in (rxdata_n),
    .tx_reset_done (), 
    .rx_reset_done (gtx_rxrstdone_s), 
    .txoutclk (gtx_rxclk_s),
    .rxoutclk (),
    .txusrclk (rx_clk), 
    .txusrclkrdy (1'b1), 
    .rxusrclk (rx_clk), 
    .rxusrclkrdy (1'b1), 
    .txdata (64'd0),
    .txcharisk (8'd0),
    .rxdata (gtx_rxdata_s),
    .rxcharisk (gtx_rxcharisk_s),
    .rxdisperr (gtx_rxdisperr_s),
    .rxnotintable (gtx_rxnotintable_s),
    .rxencommaalign (gtx_rxencommaalign_s),
    .rxenchanbond (gtx_rxenchanbond_s),
    .rxchanbondseq (gtx_rxchanbondseq_s),
    .drpclk (up_clk),
    .gt0_drpaddr (up_drp_addr), 
    .gt0_drpdi (up_drp_wdata),   
    .gt0_drpen (up_drp_en_0),    
    .gt0_drpwe (up_drp_we),
    .gt0_drpdo (up_drp_rdata_0_s),
    .gt0_drprdy (up_drp_ready_0_s),
    .gt1_drpaddr (up_drp_addr), 
    .gt1_drpdi (up_drp_wdata),   
    .gt1_drpen (up_drp_en_1),    
    .gt1_drpwe (up_drp_we),
    .gt1_drpdo (up_drp_rdata_1_s),
    .gt1_drprdy (up_drp_ready_1_s));
  // GTX eye scan (per GTX)

  cf_gtx_es_if i_gtx_es_if (
    .up_rstn (up_rstn),
    .up_clk (up_clk),
    .up_start (up_es_start),
    .up_mode (up_es_mode),
    .up_prescale (up_prescale),
    .up_hoffset_min (up_hoffset_min),
    .up_hoffset_max (up_hoffset_max),
    .up_hoffset_step (up_hoffset_step),
    .up_voffset_min (up_voffset_min),
    .up_voffset_max (up_voffset_max),
    .up_voffset_step (up_voffset_step),
    .up_drp_en (up_drp_en_s),
    .up_drp_we (up_drp_we_s),
    .up_drp_addr (up_drp_addr_s),
    .up_drp_wdata (up_drp_wdata_s),
    .up_drp_rdata (up_drp_rdata),
    .up_drp_ready (up_drp_ready),
    .es_valid (es_valid_s),
    .es_sos (es_sos_s),
    .es_eos (es_eos_s),
    .es_data (es_data_s),
    .es_state (es_state_s),
    .es_dbg_data (es_dbg_data),
    .es_dbg_trigger (es_dbg_trigger));

  // Eye scan write for master

  cf_gtx_es_wr i_gtx_es_wr (
    .up_rstn (up_rstn),
    .up_clk (up_clk),
    .up_startaddr (up_startaddr),
    .up_hsize (up_hsize),
    .up_hmin (up_hmin),
    .up_hmax (up_hmax),
    .es_valid (es_valid_s),
    .es_sos (es_sos_s),
    .es_eos (es_eos_s),
    .es_data (es_data_s),
    .mb_req (mb_req),
    .mb_ack (mb_ack),
    .mb_addr (mb_addr),
    .mb_data (mb_data),
    .mb_done (mb_done),
    .mb_error (mb_error),
    .mb_readyn (mb_readyn),
    .mb_ovf (mb_ovf_s),
    .mb_unf (mb_unf_s),
    .mb_state (mb_state_s),
    .mb_status (mb_status_s),
    .mb_dbg_data (mb_dbg_data),
    .mb_dbg_trigger (mb_dbg_trigger));

  // JESD monitor 0

  cf_jesd_mon i_mon_0 (
    .rx_rst (jesd204b_rst_s),
    .rx_clk (rx_clk),
    .rx_data (gtx_rxdata_s[31:0]),
    .rx_charisk (gtx_rxcharisk_s[3:0]),
    .rx_disperr (gtx_rxdisperr_s[3:0]),
    .rx_notintable (gtx_rxnotintable_s[3:0]),
    .rx_sysref (rxsysref),
    .rx_sync (rxsync),
    .rx_sysref_enb (rx_sysref_enb_s[0]),
    .rx_sync_enb (rx_sync_enb_s[0]),
    .rx_fsm (rx_fsm_s[3:0]),
    .mon_trigger (debug_trigger[3:0]),
    .mon_data (debug_data[55:0]));

  // JESD monitor 1

  cf_jesd_mon i_mon_1 (
    .rx_rst (jesd204b_rst_s),
    .rx_clk (rx_clk),
    .rx_data (gtx_rxdata_s[63:32]),
    .rx_charisk (gtx_rxcharisk_s[7:4]),
    .rx_disperr (gtx_rxdisperr_s[7:4]),
    .rx_notintable (gtx_rxnotintable_s[7:4]),
    .rx_sysref (rxsysref),
    .rx_sync (rxsync),
    .rx_sysref_enb (rx_sysref_enb_s[1]),
    .rx_sync_enb (rx_sync_enb_s[1]),
    .rx_fsm (rx_fsm_s[7:4]),
    .mon_trigger (debug_trigger[7:4]),
    .mon_data (debug_data[111:56]));


endmodule

// ***************************************************************************
// ***************************************************************************
