// ***************************************************************************
// ***************************************************************************
// Copyright 2011(c) Analog Devices, Inc.
// 
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//     - Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     - Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in
//       the documentation and/or other materials provided with the
//       distribution.
//     - Neither the name of Analog Devices, Inc. nor the names of its
//       contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//     - The use of this software may or may not infringe the patent rights
//       of one or more patent holders.  This license does not release you
//       from the requirement that you obtain separate licenses from these
//       patent holders to use this software.
//     - Use of the software either in source or binary form, must be run
//       on or directly connected to an Analog Devices Inc. component.
//    
// THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED.
//
// IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, INTELLECTUAL PROPERTY
// RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
// THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// ***************************************************************************
// ***************************************************************************
// ***************************************************************************
// ***************************************************************************

module user_logic (

  ref_clk_in,
  rxdata_p,
  rxdata_n,
  rxsync,
  rxsysref,

  rx_clk,
  rx_data0,
  rx_data1,

  es_dbg_data,
  es_dbg_trigger,

  mb_dbg_data,
  mb_dbg_trigger,

  debug_data,
  debug_trigger,

  Bus2IP_Clk,
  Bus2IP_Resetn,
  Bus2IP_Data,
  Bus2IP_BE,
  Bus2IP_RdCE,
  Bus2IP_WrCE,
  IP2Bus_Data,
  IP2Bus_RdAck,
  IP2Bus_WrAck,
  IP2Bus_Error,
  ip2bus_mstrd_req,
  ip2bus_mstwr_req,
  ip2bus_mst_addr,
  ip2bus_mst_be,
  ip2bus_mst_lock,
  ip2bus_mst_reset,
  bus2ip_mst_cmdack,
  bus2ip_mst_cmplt,
  bus2ip_mst_error,
  bus2ip_mst_rearbitrate,
  bus2ip_mst_cmd_timeout,
  bus2ip_mstrd_d,
  bus2ip_mstrd_src_rdy_n,
  ip2bus_mstwr_d,
  bus2ip_mstwr_dst_rdy_n);

  parameter C_MST_AWIDTH = 32;
  parameter C_MST_DWIDTH = 32;
  parameter C_NUM_REG = 32;
  parameter C_SLV_DWIDTH = 32;
  parameter C_CF_BUFTYPE = 0;

  input           ref_clk_in;
  input   [ 1:0]  rxdata_p;
  input   [ 1:0]  rxdata_n;
  output          rxsync;
  output          rxsysref;

  output          rx_clk;
  output  [31:0]  rx_data0;
  output  [31:0]  rx_data1;

  output [227:0]  es_dbg_data;
  output  [ 7:0]  es_dbg_trigger;

  output  [83:0]  mb_dbg_data;
  output  [ 7:0]  mb_dbg_trigger;

  output [199:0]  debug_data;
  output  [ 9:0]  debug_trigger;

  input           Bus2IP_Clk;
  input           Bus2IP_Resetn;
  input   [31:0]  Bus2IP_Data;
  input   [ 3:0]  Bus2IP_BE;
  input   [35:0]  Bus2IP_RdCE;
  input   [35:0]  Bus2IP_WrCE;
  output  [31:0]  IP2Bus_Data;
  output          IP2Bus_RdAck;
  output          IP2Bus_WrAck;
  output          IP2Bus_Error;
  output          ip2bus_mstrd_req;
  output          ip2bus_mstwr_req;
  output  [31: 0] ip2bus_mst_addr;
  output  [ 3: 0] ip2bus_mst_be;
  output          ip2bus_mst_lock;
  output          ip2bus_mst_reset;
  input           bus2ip_mst_cmdack;
  input           bus2ip_mst_cmplt;
  input           bus2ip_mst_error;
  input           bus2ip_mst_rearbitrate;
  input           bus2ip_mst_cmd_timeout;
  input   [31: 0] bus2ip_mstrd_d;
  input           bus2ip_mstrd_src_rdy_n;
  output  [31: 0] ip2bus_mstwr_d;
  input           bus2ip_mstwr_dst_rdy_n;

  reg             up_sel;
  reg             up_rwn;
  reg     [ 4:0]  up_addr;
  reg     [31:0]  up_wdata;
  reg             IP2Bus_RdAck;
  reg             IP2Bus_WrAck;
  reg     [31:0]  IP2Bus_Data;
  reg             IP2Bus_Error;

  wire    [35:0]  up_rwce_s;
  wire    [31:0]  up_rdata_s;
  wire            up_ack_s;

  assign up_rwce_s = (Bus2IP_RdCE == 0) ? Bus2IP_WrCE : Bus2IP_RdCE;

  always @(negedge Bus2IP_Resetn or posedge Bus2IP_Clk) begin
    if (Bus2IP_Resetn == 0) begin
      up_sel <= 'd0;
      up_rwn <= 'd0;
      up_addr <= 'd0;
      up_wdata <= 'd0;
    end else begin
      up_sel <= (up_rwce_s == 0) ? 1'b0 : 1'b1;
      up_rwn <= (Bus2IP_RdCE == 0) ? 1'b0 : 1'b1;
      case (up_rwce_s)
        36'h800000000: up_addr <= 5'h00;
        36'h400000000: up_addr <= 5'h01;
        36'h200000000: up_addr <= 5'h02;
        36'h100000000: up_addr <= 5'h03;
        36'h080000000: up_addr <= 5'h04;
        36'h040000000: up_addr <= 5'h05;
        36'h020000000: up_addr <= 5'h06;
        36'h010000000: up_addr <= 5'h07;
        36'h008000000: up_addr <= 5'h08;
        36'h004000000: up_addr <= 5'h09;
        36'h002000000: up_addr <= 5'h0a;
        36'h001000000: up_addr <= 5'h0b;
        36'h000800000: up_addr <= 5'h0c;
        36'h000400000: up_addr <= 5'h0d;
        36'h000200000: up_addr <= 5'h0e;
        36'h000100000: up_addr <= 5'h0f;
        36'h000080000: up_addr <= 5'h10;
        36'h000040000: up_addr <= 5'h11;
        36'h000020000: up_addr <= 5'h12;
        36'h000010000: up_addr <= 5'h13;
        36'h000008000: up_addr <= 5'h14;
        36'h000004000: up_addr <= 5'h15;
        36'h000002000: up_addr <= 5'h16;
        36'h000001000: up_addr <= 5'h17;
        36'h000000800: up_addr <= 5'h18;
        36'h000000400: up_addr <= 5'h19;
        36'h000000200: up_addr <= 5'h1a;
        36'h000000100: up_addr <= 5'h1b;
        36'h000000080: up_addr <= 5'h1c;
        36'h000000040: up_addr <= 5'h1d;
        36'h000000020: up_addr <= 5'h1e;
        36'h000000010: up_addr <= 5'h1f;
        default: up_addr <= 5'h1f;
      endcase
      up_wdata <= Bus2IP_Data;
    end
  end

  always @(negedge Bus2IP_Resetn or posedge Bus2IP_Clk) begin
    if (Bus2IP_Resetn == 0) begin
      IP2Bus_RdAck <= 'd0;
      IP2Bus_WrAck <= 'd0;
      IP2Bus_Data <= 'd0;
      IP2Bus_Error <= 'd0;
    end else begin
      IP2Bus_RdAck <= (Bus2IP_RdCE == 0) ? 1'b0 : up_ack_s;
      IP2Bus_WrAck <= (Bus2IP_WrCE == 0) ? 1'b0 : up_ack_s;
      IP2Bus_Data <= up_rdata_s;
      IP2Bus_Error <= 'd0;
    end
  end

  assign ip2bus_mstrd_req = 1'b0;
  assign ip2bus_mst_be = 4'hf;
  assign ip2bus_mst_lock = 1'b0;
  assign ip2bus_mst_reset = 1'b0;

  jesd204b_rx2_top #(.C_CF_BUFTYPE(C_CF_BUFTYPE)) i_jesd204b_rx2_top (
    .ref_clk_in (ref_clk_in),
    .rxdata_p (rxdata_p),
    .rxdata_n (rxdata_n),
    .rxsync (rxsync),
    .rxsysref (rxsysref),
    .rx_clk (rx_clk),
    .rx_data0 (rx_data0),
    .rx_data1 (rx_data1),
    .mb_req (ip2bus_mstwr_req),
    .mb_ack (bus2ip_mst_cmdack),
    .mb_addr (ip2bus_mst_addr),
    .mb_data (ip2bus_mstwr_d),
    .mb_done (bus2ip_mst_cmplt),
    .mb_error (bus2ip_mst_error),
    .mb_readyn (bus2ip_mstwr_dst_rdy_n),
    .up_rstn (Bus2IP_Resetn),
    .up_clk (Bus2IP_Clk),
    .up_sel (up_sel),
    .up_rwn (up_rwn),
    .up_addr (up_addr),
    .up_wdata (up_wdata),
    .up_rdata (up_rdata_s),
    .up_ack (up_ack_s),
    .es_dbg_data (es_dbg_data),
    .es_dbg_trigger (es_dbg_trigger),
    .mb_dbg_data (mb_dbg_data),
    .mb_dbg_trigger (mb_dbg_trigger),
    .debug_data (debug_data),
    .debug_trigger (debug_trigger));

endmodule

// ***************************************************************************
// ***************************************************************************
