-- ***************************************************************************
-- ***************************************************************************
-- ***************************************************************************
-- ***************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library proc_common_v3_00_a;
use proc_common_v3_00_a.proc_common_pkg.all;
use proc_common_v3_00_a.ipif_pkg.all;

library axi_lite_ipif_v1_01_a;
use axi_lite_ipif_v1_01_a.axi_lite_ipif;

library axi_master_lite_v1_00_a;
use axi_master_lite_v1_00_a.axi_master_lite;

entity axi_jesd204b_rx2 is
  generic
  (
    C_S_AXI_DATA_WIDTH : integer := 32;
    C_S_AXI_ADDR_WIDTH : integer := 32;
    C_S_AXI_MIN_SIZE : std_logic_vector := X"000001FF";
    C_USE_WSTRB : integer := 0;
    C_DPHASE_TIMEOUT : integer := 8;
    C_BASEADDR : std_logic_vector := X"FFFFFFFF";
    C_HIGHADDR : std_logic_vector := X"00000000";
    C_FAMILY : string := "virtex6";
    C_NUM_REG : integer := 32;
    C_NUM_MEM : integer := 1;
    C_SLV_AWIDTH : integer := 32;
    C_SLV_DWIDTH : integer := 32;
    C_M_AXI_LITE_ADDR_WIDTH : integer := 32;
    C_M_AXI_LITE_DATA_WIDTH : integer := 32;
    C_CF_BUFTYPE : integer := 0
  );
  port
  (
    ref_clk_in : in  std_logic;
    rxdata_p : in  std_logic_vector(1 downto 0);
    rxdata_n : in  std_logic_vector(1 downto 0);
    rxsync : out std_logic;
    rxsysref : out std_logic;
    rx_clk : out std_logic;
    rx_data0 : out std_logic_vector(31 downto 0);
    rx_data1 : out std_logic_vector(31 downto 0);
    es_dbg_data : out std_logic_vector(227 downto 0); 
    es_dbg_trigger : out std_logic_vector(7 downto 0);
    mb_dbg_data : out std_logic_vector(83 downto 0);
    mb_dbg_trigger : out std_logic_vector(7 downto 0);  
    debug_data : out std_logic_vector(199 downto 0);
    debug_trigger : out std_logic_vector(9 downto 0);
    S_AXI_ACLK : in  std_logic;
    S_AXI_ARESETN : in  std_logic;
    S_AXI_AWADDR : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
    S_AXI_AWVALID : in  std_logic;
    S_AXI_WDATA : in  std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
    S_AXI_WSTRB : in  std_logic_vector((C_S_AXI_DATA_WIDTH/8)-1 downto 0);
    S_AXI_WVALID : in  std_logic;
    S_AXI_BREADY : in  std_logic;
    S_AXI_ARADDR : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
    S_AXI_ARVALID : in  std_logic;
    S_AXI_RREADY : in  std_logic;
    S_AXI_ARREADY : out std_logic;
    S_AXI_RDATA : out std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
    S_AXI_RRESP : out std_logic_vector(1 downto 0);
    S_AXI_RVALID : out std_logic;
    S_AXI_WREADY : out std_logic;
    S_AXI_BRESP : out std_logic_vector(1 downto 0);
    S_AXI_BVALID : out std_logic;
    S_AXI_AWREADY : out std_logic;
    m_axi_lite_aclk : in  std_logic;
    m_axi_lite_aresetn : in  std_logic;
    md_error : out std_logic;
    m_axi_lite_arready : in  std_logic;
    m_axi_lite_arvalid : out std_logic;
    m_axi_lite_araddr : out std_logic_vector(C_M_AXI_LITE_ADDR_WIDTH-1 downto 0);
    m_axi_lite_arprot : out std_logic_vector(2 downto 0);
    m_axi_lite_rready : out std_logic;
    m_axi_lite_rvalid : in  std_logic;
    m_axi_lite_rdata : in  std_logic_vector(C_M_AXI_LITE_DATA_WIDTH-1 downto 0);
    m_axi_lite_rresp : in  std_logic_vector(1 downto 0);
    m_axi_lite_awready : in  std_logic;
    m_axi_lite_awvalid : out std_logic;
    m_axi_lite_awaddr : out std_logic_vector(C_M_AXI_LITE_ADDR_WIDTH-1 downto 0);
    m_axi_lite_awprot : out std_logic_vector(2 downto 0);
    m_axi_lite_wready : in  std_logic;
    m_axi_lite_wvalid : out std_logic;
    m_axi_lite_wdata : out std_logic_vector(C_M_AXI_LITE_DATA_WIDTH-1 downto 0);
    m_axi_lite_wstrb : out std_logic_vector((C_M_AXI_LITE_DATA_WIDTH/8)-1 downto 0);
    m_axi_lite_bready : out std_logic;
    m_axi_lite_bvalid : in  std_logic;
    m_axi_lite_bresp : in  std_logic_vector(1 downto 0)
  );

  attribute MAX_FANOUT : string;
  attribute SIGIS : string;
  attribute MAX_FANOUT of S_AXI_ACLK : signal is "10000";
  attribute MAX_FANOUT of S_AXI_ARESETN : signal is "10000";
  attribute SIGIS of S_AXI_ACLK : signal is "Clk";
  attribute SIGIS of S_AXI_ARESETN : signal is "Rst";
  attribute MAX_FANOUT of m_axi_lite_aclk : signal is "10000";
  attribute MAX_FANOUT of m_axi_lite_aresetn : signal is "10000";
  attribute SIGIS of m_axi_lite_aclk : signal is "Clk";
  attribute SIGIS of m_axi_lite_aresetn : signal is "Rst";
end entity axi_jesd204b_rx2;

architecture IMP of axi_jesd204b_rx2 is

  constant USER_SLV_DWIDTH : integer := C_S_AXI_DATA_WIDTH;
  constant IPIF_SLV_DWIDTH : integer := C_S_AXI_DATA_WIDTH;
  constant ZERO_ADDR_PAD : std_logic_vector(0 to 31) := (others => '0');
  constant USER_SLV_BASEADDR : std_logic_vector := C_BASEADDR or X"00000000";
  constant USER_SLV_HIGHADDR : std_logic_vector := C_BASEADDR or X"000000FF";
  constant USER_MST_BASEADDR : std_logic_vector := C_BASEADDR or X"00000100";
  constant USER_MST_HIGHADDR : std_logic_vector := C_BASEADDR or X"000001FF";
  constant IPIF_ARD_ADDR_RANGE_ARRAY : SLV64_ARRAY_TYPE := 
    (ZERO_ADDR_PAD & USER_SLV_BASEADDR, ZERO_ADDR_PAD & USER_SLV_HIGHADDR,
    ZERO_ADDR_PAD & USER_MST_BASEADDR, ZERO_ADDR_PAD & USER_MST_HIGHADDR);
  constant USER_SLV_NUM_REG : integer := 32;
  constant USER_MST_NUM_REG : integer := 4;
  constant USER_NUM_REG : integer := USER_SLV_NUM_REG+USER_MST_NUM_REG;
  constant TOTAL_IPIF_CE : integer := USER_NUM_REG;
  constant IPIF_ARD_NUM_CE_ARRAY : INTEGER_ARRAY_TYPE := (0  => (USER_SLV_NUM_REG), 1  => (USER_MST_NUM_REG));
  constant USER_MST_AWIDTH : integer := C_M_AXI_LITE_ADDR_WIDTH;
  constant USER_MST_DWIDTH : integer := C_M_AXI_LITE_DATA_WIDTH;
  constant USER_SLV_CS_INDEX : integer := 0;
  constant USER_SLV_CE_INDEX : integer := calc_start_ce_index(IPIF_ARD_NUM_CE_ARRAY, USER_SLV_CS_INDEX);
  constant USER_MST_CS_INDEX : integer := 1;
  constant USER_MST_CE_INDEX : integer := calc_start_ce_index(IPIF_ARD_NUM_CE_ARRAY, USER_MST_CS_INDEX);
  constant USER_CE_INDEX : integer := USER_SLV_CE_INDEX;

  signal ipif_Bus2IP_Clk : std_logic;
  signal ipif_Bus2IP_Resetn : std_logic;
  signal ipif_Bus2IP_Addr : std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
  signal ipif_Bus2IP_RNW : std_logic;
  signal ipif_Bus2IP_BE : std_logic_vector(IPIF_SLV_DWIDTH/8-1 downto 0);
  signal ipif_Bus2IP_CS : std_logic_vector((IPIF_ARD_ADDR_RANGE_ARRAY'LENGTH)/2-1 downto 0);
  signal ipif_Bus2IP_RdCE : std_logic_vector(calc_num_ce(IPIF_ARD_NUM_CE_ARRAY)-1 downto 0);
  signal ipif_Bus2IP_WrCE : std_logic_vector(calc_num_ce(IPIF_ARD_NUM_CE_ARRAY)-1 downto 0);
  signal ipif_Bus2IP_Data : std_logic_vector(IPIF_SLV_DWIDTH-1 downto 0);
  signal ipif_IP2Bus_WrAck : std_logic;
  signal ipif_IP2Bus_RdAck : std_logic;
  signal ipif_IP2Bus_Error : std_logic;
  signal ipif_IP2Bus_Data : std_logic_vector(IPIF_SLV_DWIDTH-1 downto 0);
  signal ipif_ip2bus_mstrd_req : std_logic;
  signal ipif_ip2bus_mstwr_req : std_logic;
  signal ipif_ip2bus_mst_addr : std_logic_vector(0 to C_M_AXI_LITE_ADDR_WIDTH-1);
  signal ipif_ip2bus_mst_be : std_logic_vector(0 to (C_M_AXI_LITE_DATA_WIDTH/8)-1);
  signal ipif_ip2bus_mst_lock : std_logic;
  signal ipif_ip2bus_mst_reset : std_logic;
  signal ipif_bus2ip_mst_cmdack : std_logic;
  signal ipif_bus2ip_mst_cmplt : std_logic;
  signal ipif_bus2ip_mst_error : std_logic;
  signal ipif_bus2ip_mst_rearbitrate : std_logic;
  signal ipif_bus2ip_mst_cmd_timeout : std_logic;
  signal ipif_bus2ip_mstrd_d : std_logic_vector(0 to C_M_AXI_LITE_DATA_WIDTH-1);
  signal ipif_bus2ip_mstrd_src_rdy_n : std_logic;
  signal ipif_ip2bus_mstwr_d : std_logic_vector(0 to C_M_AXI_LITE_DATA_WIDTH-1);
  signal ipif_bus2ip_mstwr_dst_rdy_n : std_logic;
  signal user_Bus2IP_RdCE : std_logic_vector(USER_NUM_REG-1 downto 0);
  signal user_Bus2IP_WrCE : std_logic_vector(USER_NUM_REG-1 downto 0);
  signal user_IP2Bus_Data : std_logic_vector(USER_SLV_DWIDTH-1 downto 0);
  signal user_IP2Bus_RdAck : std_logic;
  signal user_IP2Bus_WrAck : std_logic;
  signal user_IP2Bus_Error : std_logic;

  component user_logic is
    generic
    (
      C_MST_AWIDTH : integer := 32;
      C_MST_DWIDTH : integer := 32;
      C_NUM_REG : integer := 36;
      C_SLV_DWIDTH : integer := 32;
      C_CF_BUFTYPE : integer := 0
    );
    port
    (
      ref_clk_in : in  std_logic;
      rxdata_p : in  std_logic_vector(1 downto 0);
      rxdata_n : in  std_logic_vector(1 downto 0);
      rxsync : out std_logic;
      rxsysref : out std_logic;
      rx_clk : out std_logic;
      rx_data0 : out std_logic_vector(31 downto 0);
      rx_data1 : out std_logic_vector(31 downto 0);
      es_dbg_data : out std_logic_vector(227 downto 0); 
      es_dbg_trigger : out std_logic_vector(7 downto 0);
      mb_dbg_data : out std_logic_vector(83 downto 0);
      mb_dbg_trigger : out std_logic_vector(7 downto 0);  
      debug_data : out std_logic_vector(199 downto 0);
      debug_trigger : out std_logic_vector(9 downto 0);
      Bus2IP_Clk : in  std_logic;
      Bus2IP_Resetn : in  std_logic;
      Bus2IP_Data : in  std_logic_vector(C_SLV_DWIDTH-1 downto 0);
      Bus2IP_BE : in  std_logic_vector(C_SLV_DWIDTH/8-1 downto 0);
      Bus2IP_RdCE : in  std_logic_vector(C_NUM_REG-1 downto 0);
      Bus2IP_WrCE : in  std_logic_vector(C_NUM_REG-1 downto 0);
      IP2Bus_Data : out std_logic_vector(C_SLV_DWIDTH-1 downto 0);
      IP2Bus_RdAck : out std_logic;
      IP2Bus_WrAck : out std_logic;
      IP2Bus_Error : out std_logic;
      ip2bus_mstrd_req : out std_logic;
      ip2bus_mstwr_req : out std_logic;
      ip2bus_mst_addr : out std_logic_vector(C_MST_AWIDTH-1 downto 0);
      ip2bus_mst_be : out std_logic_vector((C_MST_DWIDTH/8)-1 downto 0);
      ip2bus_mst_lock : out std_logic;
      ip2bus_mst_reset : out std_logic;
      bus2ip_mst_cmdack : in  std_logic;
      bus2ip_mst_cmplt : in  std_logic;
      bus2ip_mst_error : in  std_logic;
      bus2ip_mst_rearbitrate : in  std_logic;
      bus2ip_mst_cmd_timeout : in  std_logic;
      bus2ip_mstrd_d : in  std_logic_vector(C_MST_DWIDTH-1 downto 0);
      bus2ip_mstrd_src_rdy_n : in  std_logic;
      ip2bus_mstwr_d : out std_logic_vector(C_MST_DWIDTH-1 downto 0);
      bus2ip_mstwr_dst_rdy_n : in  std_logic
    );
  end component user_logic;

begin

  AXI_LITE_IPIF_I : entity axi_lite_ipif_v1_01_a.axi_lite_ipif
    generic map
    (
      C_S_AXI_DATA_WIDTH => IPIF_SLV_DWIDTH,
      C_S_AXI_ADDR_WIDTH => C_S_AXI_ADDR_WIDTH,
      C_S_AXI_MIN_SIZE => C_S_AXI_MIN_SIZE,
      C_USE_WSTRB => C_USE_WSTRB,
      C_DPHASE_TIMEOUT => C_DPHASE_TIMEOUT,
      C_ARD_ADDR_RANGE_ARRAY => IPIF_ARD_ADDR_RANGE_ARRAY,
      C_ARD_NUM_CE_ARRAY => IPIF_ARD_NUM_CE_ARRAY,
      C_FAMILY => C_FAMILY
    )
    port map
    (
      S_AXI_ACLK => S_AXI_ACLK,
      S_AXI_ARESETN => S_AXI_ARESETN,
      S_AXI_AWADDR => S_AXI_AWADDR,
      S_AXI_AWVALID => S_AXI_AWVALID,
      S_AXI_WDATA => S_AXI_WDATA,
      S_AXI_WSTRB => S_AXI_WSTRB,
      S_AXI_WVALID => S_AXI_WVALID,
      S_AXI_BREADY => S_AXI_BREADY,
      S_AXI_ARADDR => S_AXI_ARADDR,
      S_AXI_ARVALID => S_AXI_ARVALID,
      S_AXI_RREADY => S_AXI_RREADY,
      S_AXI_ARREADY => S_AXI_ARREADY,
      S_AXI_RDATA => S_AXI_RDATA,
      S_AXI_RRESP => S_AXI_RRESP,
      S_AXI_RVALID => S_AXI_RVALID,
      S_AXI_WREADY => S_AXI_WREADY,
      S_AXI_BRESP => S_AXI_BRESP,
      S_AXI_BVALID => S_AXI_BVALID,
      S_AXI_AWREADY => S_AXI_AWREADY,
      Bus2IP_Clk => ipif_Bus2IP_Clk,
      Bus2IP_Resetn => ipif_Bus2IP_Resetn,
      Bus2IP_Addr => ipif_Bus2IP_Addr,
      Bus2IP_RNW => ipif_Bus2IP_RNW,
      Bus2IP_BE => ipif_Bus2IP_BE,
      Bus2IP_CS => ipif_Bus2IP_CS,
      Bus2IP_RdCE => ipif_Bus2IP_RdCE,
      Bus2IP_WrCE => ipif_Bus2IP_WrCE,
      Bus2IP_Data => ipif_Bus2IP_Data,
      IP2Bus_WrAck => ipif_IP2Bus_WrAck,
      IP2Bus_RdAck => ipif_IP2Bus_RdAck,
      IP2Bus_Error => ipif_IP2Bus_Error,
      IP2Bus_Data => ipif_IP2Bus_Data
    );

  AXI_MASTER_LITE_I : entity axi_master_lite_v1_00_a.axi_master_lite
    generic map
    (
      C_M_AXI_LITE_ADDR_WIDTH => C_M_AXI_LITE_ADDR_WIDTH,
      C_M_AXI_LITE_DATA_WIDTH => C_M_AXI_LITE_DATA_WIDTH,
      C_FAMILY => C_FAMILY
    )
    port map
    (
      m_axi_lite_aclk => m_axi_lite_aclk,
      m_axi_lite_aresetn => m_axi_lite_aresetn,
      md_error => md_error,
      m_axi_lite_arready => m_axi_lite_arready,
      m_axi_lite_arvalid => m_axi_lite_arvalid,
      m_axi_lite_araddr => m_axi_lite_araddr,
      m_axi_lite_arprot => m_axi_lite_arprot,
      m_axi_lite_rready => m_axi_lite_rready,
      m_axi_lite_rvalid => m_axi_lite_rvalid,
      m_axi_lite_rdata => m_axi_lite_rdata,
      m_axi_lite_rresp => m_axi_lite_rresp,
      m_axi_lite_awready => m_axi_lite_awready,
      m_axi_lite_awvalid => m_axi_lite_awvalid,
      m_axi_lite_awaddr => m_axi_lite_awaddr,
      m_axi_lite_awprot => m_axi_lite_awprot,
      m_axi_lite_wready => m_axi_lite_wready,
      m_axi_lite_wvalid => m_axi_lite_wvalid,
      m_axi_lite_wdata => m_axi_lite_wdata,
      m_axi_lite_wstrb => m_axi_lite_wstrb,
      m_axi_lite_bready => m_axi_lite_bready,
      m_axi_lite_bvalid => m_axi_lite_bvalid,
      m_axi_lite_bresp => m_axi_lite_bresp,
      ip2bus_mstrd_req => ipif_ip2bus_mstrd_req,
      ip2bus_mstwr_req => ipif_ip2bus_mstwr_req,
      ip2bus_mst_addr => ipif_ip2bus_mst_addr,
      ip2bus_mst_be => ipif_ip2bus_mst_be,
      ip2bus_mst_lock => ipif_ip2bus_mst_lock,
      ip2bus_mst_reset => ipif_ip2bus_mst_reset,
      bus2ip_mst_cmdack => ipif_bus2ip_mst_cmdack,
      bus2ip_mst_cmplt => ipif_bus2ip_mst_cmplt,
      bus2ip_mst_error => ipif_bus2ip_mst_error,
      bus2ip_mst_rearbitrate => ipif_bus2ip_mst_rearbitrate,
      bus2ip_mst_cmd_timeout => ipif_bus2ip_mst_cmd_timeout,
      bus2ip_mstrd_d => ipif_bus2ip_mstrd_d,
      bus2ip_mstrd_src_rdy_n => ipif_bus2ip_mstrd_src_rdy_n,
      ip2bus_mstwr_d => ipif_ip2bus_mstwr_d,
      bus2ip_mstwr_dst_rdy_n => ipif_bus2ip_mstwr_dst_rdy_n
    );

  USER_LOGIC_I : component user_logic
    generic map
    (
      C_MST_AWIDTH => USER_MST_AWIDTH,
      C_MST_DWIDTH => USER_MST_DWIDTH,
      C_NUM_REG => USER_NUM_REG,
      C_SLV_DWIDTH => USER_SLV_DWIDTH,
      C_CF_BUFTYPE => C_CF_BUFTYPE
    )
    port map
    (
      ref_clk_in => ref_clk_in,
      rxdata_p => rxdata_p,
      rxdata_n => rxdata_n,
      rxsync => rxsync,
      rxsysref => rxsysref,
      rx_clk => rx_clk,
      rx_data0 => rx_data0,
      rx_data1 => rx_data1,
      es_dbg_data => es_dbg_data,
      es_dbg_trigger => es_dbg_trigger,
      mb_dbg_data => mb_dbg_data,
      mb_dbg_trigger => mb_dbg_trigger,
      debug_data => debug_data,
      debug_trigger => debug_trigger,
      Bus2IP_Clk => ipif_Bus2IP_Clk,
      Bus2IP_Resetn => ipif_Bus2IP_Resetn,
      Bus2IP_Data => ipif_Bus2IP_Data,
      Bus2IP_BE => ipif_Bus2IP_BE,
      Bus2IP_RdCE => user_Bus2IP_RdCE,
      Bus2IP_WrCE => user_Bus2IP_WrCE,
      IP2Bus_Data => user_IP2Bus_Data,
      IP2Bus_RdAck => user_IP2Bus_RdAck,
      IP2Bus_WrAck => user_IP2Bus_WrAck,
      IP2Bus_Error => user_IP2Bus_Error,
      ip2bus_mstrd_req => ipif_ip2bus_mstrd_req,
      ip2bus_mstwr_req => ipif_ip2bus_mstwr_req,
      ip2bus_mst_addr => ipif_ip2bus_mst_addr,
      ip2bus_mst_be => ipif_ip2bus_mst_be,
      ip2bus_mst_lock => ipif_ip2bus_mst_lock,
      ip2bus_mst_reset => ipif_ip2bus_mst_reset,
      bus2ip_mst_cmdack => ipif_bus2ip_mst_cmdack,
      bus2ip_mst_cmplt => ipif_bus2ip_mst_cmplt,
      bus2ip_mst_error => ipif_bus2ip_mst_error,
      bus2ip_mst_rearbitrate => ipif_bus2ip_mst_rearbitrate,
      bus2ip_mst_cmd_timeout => ipif_bus2ip_mst_cmd_timeout,
      bus2ip_mstrd_d => ipif_bus2ip_mstrd_d,
      bus2ip_mstrd_src_rdy_n => ipif_bus2ip_mstrd_src_rdy_n,
      ip2bus_mstwr_d => ipif_ip2bus_mstwr_d,
      bus2ip_mstwr_dst_rdy_n => ipif_bus2ip_mstwr_dst_rdy_n);

  ipif_IP2Bus_Data <= user_IP2Bus_Data;
  ipif_IP2Bus_WrAck <= user_IP2Bus_WrAck;
  ipif_IP2Bus_RdAck <= user_IP2Bus_RdAck;
  ipif_IP2Bus_Error <= user_IP2Bus_Error;
  user_Bus2IP_RdCE <= ipif_Bus2IP_RdCE(USER_NUM_REG-1 downto 0);
  user_Bus2IP_WrCE <= ipif_Bus2IP_WrCE(USER_NUM_REG-1 downto 0);

end IMP;

-- ***************************************************************************
-- ***************************************************************************
