-- ***************************************************************************
-- ***************************************************************************
-- Copyright 2011(c) Analog Devices, Inc.
-- 
-- All rights reserved.
-- 
-- Redistribution and use in source and binary forms, with or without modification,
-- are permitted provided that the following conditions are met:
--     - Redistributions of source code must retain the above copyright
--       notice, this list of conditions and the following disclaimer.
--     - Redistributions in binary form must reproduce the above copyright
--       notice, this list of conditions and the following disclaimer in
--       the documentation and/or other materials provided with the
--       distribution.
--     - Neither the name of Analog Devices, Inc. nor the names of its
--       contributors may be used to endorse or promote products derived
--       from this software without specific prior written permission.
--     - The use of this software may or may not infringe the patent rights
--       of one or more patent holders.  This license does not release you
--       from the requirement that you obtain separate licenses from these
--       patent holders to use this software.
--     - Use of the software either in source or binary form, must be run
--       on or directly connected to an Analog Devices Inc. component.
--    
-- THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-- INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
-- PARTICULAR PURPOSE ARE DISCLAIMED.
--
-- IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-- EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, INTELLECTUAL PROPERTY
-- RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
-- BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
-- STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
-- THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- ***************************************************************************
-- ***************************************************************************
-- This is the jesd front end buffer

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library unisim;
use unisim.all;

entity util_jesdbuf is
  generic (
    C_CF_BUFTYPE      : integer := 0 -- cmos (0x1) or lvds (0x0) buffer select
  );
  port (
    clk_in_p          : in  std_logic;  -- reference clock inputs
    clk_in_n          : in  std_logic;
    ref_clk_in        : out std_logic;  -- unbuffered reference clock (for gtx)
    ref_clk           : out std_logic;  -- buffered reference clock (internal use)

    rxsync_in         : in  std_logic;  -- sync from jesd core
    rxsysref_in       : in  std_logic;  -- sysref from jesd core

    rxsync_p          : out std_logic;  -- sync output to the adc device
    rxsync_n          : out std_logic;  
    rxsysref_p        : out std_logic;  -- sysref output to the adc device
    rxsysref_n        : out std_logic
  );
end util_jesdbuf;

architecture IMP of util_jesdbuf is

  component IBUFDS_GTE2
    port (
      CEB   : in  std_ulogic;
      I     : in  std_ulogic;
      IB    : in  std_ulogic;
      O     : out std_ulogic;
      ODIV2 : out std_ulogic
    );
  end component;

  component BUFG
    port (
      I     : in  std_ulogic;
      O     : out std_ulogic
    );
  end component;

  component OBUFDS
    port (
      I     : in  std_ulogic;
      O     : out std_ulogic;
      OB    : out std_ulogic
    );
  end component;

  component OBUF
    port (
      I     : in  std_ulogic;
      O     : out std_ulogic
    );
  end component;

  signal  ref_clk_s         : std_logic;
  signal  ref_clk_div2_s    : std_logic;

begin

  -- the buffers used here are based on xilinx recommendations.

  ref_clk_in <= ref_clk_s;

  i_ibufds_gte2: IBUFDS_GTE2
    port map (
      CEB     =>  '0',
      I       =>  clk_in_p,
      IB      =>  clk_in_n,
      O       =>  ref_clk_s,
      ODIV2   =>  ref_clk_div2_s
    );

  i_bufg: BUFG
    port map (
      I       =>  ref_clk_s,
      O       =>  ref_clk
    );

  g_bufs: if (C_CF_BUFTYPE = 1) generate
  i_rxsync_buf: OBUF
    port map (
      I       =>  rxsync_in,
      O       =>  rxsync_p
    );
  i_rxsysref_buf: OBUF
    port map (
      I       =>  rxsysref_in,
      O       =>  rxsysref_p
    );
  end generate;

  g_bufds: if (C_CF_BUFTYPE = 0) generate
  i_rxsync_buf: OBUFDS
    port map (
      I       =>  rxsync_in,
      O       =>  rxsync_p,
      OB      =>  rxsync_n
    );
  i_rxsysref_buf: OBUFDS
    port map (
      I       =>  rxsysref_in,
      O       =>  rxsysref_p,
      OB      =>  rxsysref_n
    );
  end generate;

end IMP;

-- ***************************************************************************
-- ***************************************************************************
